//address.cpp Written by Paul DuBois, 9-15-95
// Use freely for non comercial purposes.

#include <owl\pch.h>
#include <owl\edit.h>
#include <owl\inputdia.h>
#include <iomanip.h>
#include "address.rh"
#include "winbrowz.h"
#include "address.h"

AddressEntry::AddressEntry(char* name, char* adr1, char* citystzip,
									char* phone, char* adr2){
  name ? strcpy(AddressLine.Name, name) : strcpy(AddressLine.Name, "");
  adr1 ? strcpy(AddressLine.Adr1, adr1) : strcpy(AddressLine.Adr1, "");
  adr2 ? strcpy(AddressLine.Adr2, adr2) : strcpy(AddressLine.Adr2, "");
  citystzip ? strcpy(AddressLine.CityStZip, citystzip) : strcpy (AddressLine.CityStZip, "");
  phone ? strcpy(AddressLine.Phone, phone) : strcpy (AddressLine.Phone, "");
}


// buffers for the windows text boxes
static char EditBuffer[ 255 + ADDRESS_BUFFER_SIZE]; // 50 for file name edit box
static AddressStruct AddressNewBuffer;

NewAddressDialog::NewAddressDialog(TWindow *parent)
:
	TDialog(parent, IDD_NEWADDRESS)
{

	EditName = new TEdit(this, IDC_NEWNAME, 40);
	EditAdr1 = new TEdit(this, IDC_NEWADR1, 40);
	EditAdr2 = new TEdit(this, IDC_NEWADR2, 40);
	EditCityStZip = new TEdit(this, IDC_NEWCITYSTZIP, 40);
	EditPhone = new TEdit(this, IDC_NEWPHONE, 40);

	SetTransferBuffer(&AddressNewBuffer);
}

DEFINE_RESPONSE_TABLE1(AddressBook, TDialog)
  EV_COMMAND(CM_VIEWTOOLS, CmViewTools),
  EV_COMMAND_ENABLE(CM_VIEWTOOLS, CeViewTools),
  EV_COMMAND(CM_BEGINNING, CmBeginning),
  EV_COMMAND_ENABLE(CM_BEGINNING, CeBeginning),
  EV_COMMAND(CM_PREVIOUS, CmPrevious),
  EV_COMMAND_ENABLE(CM_PREVIOUS, CePrevious),
  EV_COMMAND(CM_NEXT, CmNext),
  EV_COMMAND_ENABLE(CM_NEXT, CeNext),
  EV_COMMAND(CM_LAST, CmLast),
  EV_COMMAND_ENABLE(CM_LAST, CeLast),
  EV_COMMAND(IDC_HELP, BnHelp),
  EV_COMMAND(IDC_DELETE, BnDelete),
  EV_COMMAND(IDC_SEARCH, BnSearch),
  EV_COMMAND(IDC_ADD, BnAdd),
  EV_COMMAND(CM_PFILENEW, CmFileNew),
  EV_COMMAND(CM_PFILEOPEN, CmFileOpen),
  EV_COMMAND(CM_PFILESAVE, CmFileSave),
  EV_COMMAND(CM_PFILESAVEAS, CmFileSaveAs),
  EV_COMMAND(CM_ABOUT, CmAbout),

END_RESPONSE_TABLE;

AddressBook::AddressBook()
:
	TDialog (0, IDD_ADDRESSBOOK)
{
	// place to display file name
	EditFileName = new TEdit(this, IDC_FILENAME, 255);

	// Get handles to the edit boxes
	EditName = new TEdit(this, IDC_NAME, 40);
	EditAdr1 = new TEdit(this, IDC_ADR1, 40);
	EditAdr2 = new TEdit(this, IDC_ADR2, 40);
	EditCityStZip = new TEdit(this, IDC_CITYSTZIP, 40);
	EditPhone = new TEdit(this, IDC_PHONE, 40);

	SetTransferBuffer(&EditBuffer);

	// Get handles to the buttons to allow disabling them
	SearchButton = new TButton(this, IDC_SEARCH);
	DeleteButton = new TButton(this, IDC_DELETE);
	AddButton = new TButton(this, IDC_ADD);

	// create the initial address container
	List = new AddressList(10, 0, 10);

	FileData = new TOpenSaveDialog::TData(OFN_HIDEREADONLY|OFN_FILEMUSTEXIST,
												 "Address Files (*.adb)|*.adb|", "*.adb", "",
												 "adb");

	IsNewFile = true;
	IsDirty = false;

}


bool AddressBook::CanClose(){

  if (GetNumObjects())
	 CanMove(); // See if any of the edit boxes changed, and save them

  if (IsDirty)
	 switch (MessageBox("Do you want to save your changes?",
							  "Address book has changed",
							  MB_YESNOCANCEL | MB_ICONQUESTION)) {
		case IDCANCEL:
		  return false;

		case IDYES:
		  return SaveAndCheck();  // call this in order to pass back false if
										  // cancel is pressed in save as dialog for a new
										  // file
	 }
  return true; // not dirty here or no to "Save Changes?"
}

bool AddressBook::Display(){
	 if (List->GetItemsInContainer()){
		 EditName->SetText( (*List)[GetCurrent()].AddressLine.Name);
		 EditAdr1->SetText( (*List)[GetCurrent()].AddressLine.Adr1);
		 EditAdr2->SetText( (*List)[GetCurrent()].AddressLine.Adr2);
		 EditCityStZip->SetText( (*List)[GetCurrent()].AddressLine.CityStZip);
		 EditPhone->SetText( (*List)[GetCurrent()].AddressLine.Phone);
		 return true;
	 }
	 else
		return false;

}
bool AddressBook::Clear(){

	 EditName->SetText("");
	 EditAdr1->SetText("");
	 EditAdr2->SetText("");
	 EditCityStZip->SetText("");
	 EditPhone->SetText("");
	 return true;
}
bool AddressBook::CanMove(){

 if (EditName->IsModified()){
	 IsDirty = true;
	 EditName->GetText((*List)[GetCurrent()].AddressLine.Name,40);
 }
 if (EditAdr1->IsModified()){
	 IsDirty = true;
	 EditAdr1->GetText((*List)[GetCurrent()].AddressLine.Adr1,40);
 }
 if (EditAdr2->IsModified()){
	 IsDirty = true;
	 EditAdr2->GetText((*List)[GetCurrent()].AddressLine.Adr2,40);
 }
 if (EditCityStZip->IsModified()){
	 IsDirty = true;
	 EditCityStZip->GetText((*List)[GetCurrent()].AddressLine.CityStZip,40);
 }
 if (EditPhone->IsModified()){
	 IsDirty = true;
	 EditPhone->GetText((*List)[GetCurrent()].AddressLine.Phone,40);
 }

  return true;
}

void AddressBook::BnDelete(){

  CanMove();  //Check if any changes to edit boxes
  if (MessageBox("Delete this record?", "DELETE", MB_YESNO | MB_ICONQUESTION) == IDYES){
		List->Destroy(GetCurrent());
		BrowserDelete();
		IsDirty = true;
		if (List->IsEmpty()){
			SearchButton->EnableWindow(false);
			DeleteButton->EnableWindow(false);
		}
  }
}

void AddressBook::BnAdd(){

	 int x;
	 unsigned y = List->GetItemsInContainer();

	 if (y)
		CanMove();  //Check if any changes to edit boxes

	 memset(&AddressNewBuffer, 0, sizeof AddressNewBuffer);

	 if ((NewAddressDialog(this)).Execute() == IDOK){
		  IsDirty = true;
		  AddressEntry TempAddress;
		  strncpy(TempAddress.AddressLine.Name, AddressNewBuffer.Name, 40);
		  strncpy(TempAddress.AddressLine.Adr1, AddressNewBuffer.Adr1, 40);
		  strncpy(TempAddress.AddressLine.Adr2, AddressNewBuffer.Adr2, 40);
		  strncpy(TempAddress.AddressLine.CityStZip, AddressNewBuffer.CityStZip, 40);
		  strncpy(TempAddress.AddressLine.Phone, AddressNewBuffer.Phone, 40);

		  for (x = 0; x < y; x++) // Find location to insert name in alpha order
				if(strcmp(TempAddress.AddressLine.Name,(*List)[x].AddressLine.Name) <= 0)
					 break;  // found insert point when new name is =< name in list
		  List->AddAt(TempAddress, x);
		  BrowserAdd(x); // update the current
		  SearchButton->EnableWindow(true);
		  DeleteButton->EnableWindow(true);
	 }
}

void AddressBook::BnSearch() {

  int x;
  unsigned y = List->GetItemsInContainer() - 1; // Point to last object if
																// search fails

  CanMove();  //Check is any changes to edit boxes

  char inputText[31];
  memset(inputText,0,sizeof inputText);

  if ((TInputDialog(this, "SEARCH", "Name to search for:", inputText,
						  sizeof inputText)).Execute() == IDOK){
	 for (x = 0; x < y; x++) // Locate the entered name
		if(strcmp(inputText,(*List)[x].AddressLine.Name) <= 0)
		  break;  // Found a match or the closest one
	 BrowserSearch(x);
  }
}

void AddressBook::CmFileNew(){

	if (CanClose()){
		EditFileName->SetText("(New File)");
		Clear();
		AddButton->EnableWindow(true);
		List->Flush();
		IsDirty = false;
		IsNewFile = true;
		BegBrowse(0);
		SearchButton->EnableWindow(false);
		DeleteButton->EnableWindow(false);
	}
}

void AddressBook::CmFileOpen(){

  if (CanClose())
	 if (TFileOpenDialog(this, *FileData).Execute() == IDOK){
		OpenFile();
	 }
}

void AddressBook::CmFileSaveAs(){

	if (IsNewFile)
		  strcpy(FileData->FileName, "");

	if (TFileSaveDialog(this, *FileData).Execute() == IDOK)
		  SaveFile();

}


bool AddressBook::SaveAndCheck(){  // used to pass back false to can close
											  // if cancel is pressed in save as dialog

	 if (IsNewFile){
		  strcpy(FileData->FileName, "");
		  if (TFileSaveDialog(this, *FileData).Execute() == IDOK){
				SaveFile();
				return true;
		  }
		  else
				return false; // Pressed cancel in Save As dialog
	 }
	 else{
		  SaveFile();
		  return true;
	 }
}

void AddressBook::CmFileSave(){

	 if (IsNewFile)
		  CmFileSaveAs();
	 else
		  SaveFile();
}

void AddressBook::OpenFile()
{
	 ifstream is(FileData->FileName);

	 if (!is)
		  MessageBox("Unable to open file","File Error",
						 MB_OK | MB_ICONEXCLAMATION);
	 else{
		  List->Flush();
		  unsigned numAddress;
		  is >> numAddress; // get the # of address book entrys
		  is.get(); // remove the space after ItemsInContainer
		  while (numAddress--){
			 AddressEntry entry;
			 is >> entry;

			 List->Add(entry);
		  }
		  IsNewFile = IsDirty = false;
		  numAddress = List->GetItemsInContainer();
		  BegBrowse(numAddress);
		  AddButton->EnableWindow(true);
		  EditFileName->SetText(FileData->FileName);
		  if (numAddress){ // does file have entries?
			 SearchButton->EnableWindow(true);
			 DeleteButton->EnableWindow(true);
		  }
		  else
			 Clear(); // if not, clear the fields
	 }
}
istream& operator >>(istream& is, AddressEntry& entry)
{
  char ch;
  int x, y;

  char buffer [5][41];  // 5 lines of 41 chars (1 extra for possible NULL
  memset(buffer,0, sizeof buffer);

  for(x = 0; x < 5; x++)
	 for (y = 0; y < 40; y++){
		is.get(ch);
		if (ch == '\a')
		  break;
		buffer[x][y] = ch;
	 }


  strcpy (entry.AddressLine.Name, &buffer[0][0]);
  strcpy (entry.AddressLine.Adr1, &buffer[1][0]);
  strcpy (entry.AddressLine.Adr2, &buffer[2][0]);
  strcpy (entry.AddressLine.CityStZip, &buffer[3][0]);
  strcpy (entry.AddressLine.Phone, &buffer[4][0]);

  return is;
}

void AddressBook::SaveFile()
{
	 ofstream os(FileData->FileName);

	 if (!os)
		  MessageBox("Unable to open file","File Error",
						 MB_OK | MB_ICONEXCLAMATION);
	 else{
		  os << List->GetItemsInContainer() << ' ';
		  AddressIterator i(*List); // for each entry in the book
		  while (i){
			 // use '\a' to delimit fields
			 os << i.Current().AddressLine.Name << '\a';
			 os << i.Current().AddressLine.Adr1 << '\a';
			 os << i.Current().AddressLine.Adr2 << '\a';
			 os << i.Current().AddressLine.CityStZip << '\a';
			 os << i.Current().AddressLine.Phone << '\a';
			 i++;
		  }
		  IsNewFile = IsDirty = false;
		  EditFileName->SetText(FileData->FileName);
	 }
}


void AddressBook::CmAbout(){

	TDialog(this, IDD_ABOUT).Execute();
}

void AddressBook::BnHelp(){
	TDialog(this, IDD_HELP).Execute();
}
/************************************************************/


/*****************************************************************/

class TAddressBookApp : public TApplication {
  public:
	 TAddressBookApp() : TApplication ("Address Book") {}
	 void InitMainWindow();

};


void TAddressBookApp::InitMainWindow(){
  AddressBook *ABook;
  TFrameWindow* frame = new TFrameWindow(0, "Address Book", ABook = new AddressBook, true);

  //AddressBook *ABook = new AddressBook(frame);
  ABook->CreateBar(frame);   // Create the floating tool bar
  MainWindow = frame;
  MainWindow->AssignMenu(IDM_ADDRESSBOOK);
  MainWindow->Attr.AccelTable = IDM_ADDRESSBOOK;
  MainWindow->SetIcon(this, IDI_ADDRESSBOOK);

}



int
OwlMain(int /*argc*/, char* /*argv*/ [])
{
  return TAddressBookApp().Run();
}

