// Address.h  by Paul DuBois 9-15-95
// Free for non comercial use

#if !defined(ADDRESS_BOOK_H)
#define ADDRESS_BOOK_H

#if defined(BI_STD_RTL)
#include <vector>
#else
#include <owl\template.h>
#endif
#include <owl\opensave.h>
#include <owl\button.h>

//__OWL_DECLCLASS(TButton);


// Define a record format
#include <pshpack1.h>
struct AddressStruct {
	 char Name[40];
	 char Adr1[40];
	 char Adr2[40];
	 char CityStZip[40];
	 char Phone[40];
};
#include <poppack.h>

#define ADDRESS_BUFFER_SIZE 200

// Define a class to manage the records
class AddressEntry {
  public:

	 AddressEntry(char* name = 0, char* adr1 = 0,
					  char* citstzip = 0, char* phone = 0, char* adr2 = 0);

	 AddressStruct AddressLine;
	 bool operator ==(const AddressEntry& other) const
		 {return &other == this;}

};

// Create the container??
#if defined(BI_STD_RTL)
typedef std::vector<AddressEntry>           AddressList;
typedef std::vector<AddressEntry>::iterator AddressIterator;
#else
typedef TObjectArray<AddressEntry>           AddressList;
typedef TObjectArray<AddressEntry>::Iterator AddressIterator;
#endif

// use this to enter a new address book entry
class NewAddressDialog : public TDialog {
	public:
	  NewAddressDialog (TWindow *parent);
	  TEdit *EditName;
	  TEdit *EditAdr1;
	  TEdit *EditAdr2;
	  TEdit *EditCityStZip;
	  TEdit *EditPhone;
};
class AddressBook : public TDialog, public Browser {
  public:
	 AddressBook();
	 ~AddressBook(){
		  delete EditName;
		  delete EditAdr1;
		  delete EditAdr2;
		  delete EditCityStZip;
		  delete EditPhone;
		  delete SearchButton;
		  delete DeleteButton;
		  delete AddButton;
		  delete FileData;
		  delete EditFileName;
	 }

	 friend istream& operator >>(istream& is, AddressEntry& entry);

  protected:

	 AddressList *List;  // pointer to the container
	 AddressEntry transfer; // temporary holder for an address entry

	 TOpenSaveDialog::TData *FileData;
	 bool IsDirty;
	 bool IsNewFile;

	 // The dialogs edit boxes
		 TEdit* EditName;
		 TEdit* EditAdr1;
		 TEdit* EditAdr2;
		 TEdit* EditCityStZip;
		 TEdit* EditPhone;
		 TEdit* EditFileName;

	 // The dialogs buttons
		 TButton *SearchButton;
		 TButton *DeleteButton;
		 TButton *AddButton;

  ///  ** check in CanClose to see if any of the edits IsModified()
  // then save then and move true to fileisdirty

	 bool CanClose();

	 bool Display();
	 bool Clear();
	 bool CanMove();  // checks if any field has been changed

	 void BnHelp();
	 void BnDelete();
	 void BnSearch();
	 void BnAdd();

	 void CmFileNew();
	 void CmFileOpen();
	 void CmFileSave();
	 void CmFileSaveAs();

	 void CmAbout();

	 bool SaveAndCheck();
	 void OpenFile();
	 void SaveFile();

	 DECLARE_RESPONSE_TABLE(AddressBook);
};

#endif // if !defined(ADDRESS_BOOK_H
