//winbrowz.cpp
//  By Paul DuBois
//  This class implements a floating tool bar that allows browsing
//     through objects.
//  Free for non comercial use.


#include <owl\pch.h>

#include <owl\toolbox.h>
#include <owl\buttonga.h>
#include <owl\textgadg.h>
#include "address.rh"
#include "winbrowz.h"

DEFINE_RESPONSE_TABLE1(TButtonBar, TFloatingFrame)
  EV_WM_CLOSE,
  EV_COMMAND(CM_BEGINNING, CmBeginning),
  EV_COMMAND(CM_PREVIOUS, CmPrevious),
  EV_COMMAND(CM_NEXT, CmNext),
  EV_COMMAND(CM_LAST, CmLast),
  EV_COMMAND(CM_HELP, CmHelp),
END_RESPONSE_TABLE;

TButtonBar::TButtonBar(TWindow* parent, char* title, TWindow* clientWnd,
										 TModule* module)
  : TFloatingFrame(parent, title, clientWnd, true, DefaultCaptionHeight + 18, true, module)
{
  Init(parent, true);
}

void
TButtonBar::SetBarCaption(char *cs){
  SetCaption(cs);
// PaintCaption(true);
}
//
// Supply a reasonable startup position relative to our Parent (owner actually)
//

void
TButtonBar::SetupWindow()
{
  TFloatingFrame::SetupWindow();

  if (Attr.X < 0 || Attr.Y < 0) {
	 TRect parentRect;
	 Parent->GetWindowRect(parentRect);
	 SetWindowPos(0, parentRect.left+15, parentRect.top+(45 * 5), 0, 0, SWP_NOSIZE | SWP_NOZORDER);
  }
}

//
// Remember latest position when we are destroyed
//
void
TButtonBar::CleanupWindow()
{
  TRect rect;
  GetWindowRect(rect);
  Attr.X = rect.left;
  Attr.Y = rect.top;

  TFloatingFrame::CleanupWindow();
}

//
// Override EvClose to keep TWindow from having us deleted
//
void
TButtonBar::EvClose()
{
  CloseWindow();
}



void Browser::CreateBar(TFrameWindow *parent){

  TToolBox* tb = new TToolBox(0,6,1);

  tb->SetShrinkWrap(true, true);

  tb->Insert(*new TButtonGadget(CM_BEGINNING, CM_BEGINNING,
										  TButtonGadget::Command,
										  true));

  tb->Insert(*new TButtonGadget(CM_PREVIOUS, CM_PREVIOUS,
										  TButtonGadget::Command,
										  true));

  tb->Insert(*new TButtonGadget(CM_NEXT, CM_NEXT,
										  TButtonGadget::Command,
										  true));

  tb->Insert(*new TButtonGadget(CM_LAST, CM_LAST,
										  TButtonGadget::Command,
										  true));

  tb->Insert(*new TButtonGadget(CM_HELP, CM_HELP,
										  TButtonGadget::Command,
										  true));

  tb->Insert(*new TButtonGadget(CM_EXIT, CM_EXIT,
										  TButtonGadget::Command,
										  true));

  BrowseButtons = new TButtonBar(parent, "EMPTY", tb);

}

void
Browser::CmViewTools()
{
  if (BrowseButtons->HWindow)
	 BrowseButtons->Destroy();
  else
	 BrowseButtons->Create();
}

void
Browser::CeViewTools(TCommandEnabler& ce)
{
  ce.SetCheck(BrowseButtons->HWindow ?
				  TCommandEnabler::Checked : TCommandEnabler::Unchecked);
}


void Browser::BegBrowse(int NewNumObjects){

  // This function takes care of initialization

  NumObjects = NewNumObjects;
  Current = 0;
  Display();
  UpdateStatusBar();

}

void Browser::UpdateStatusBar(){

  char title[30];

  if (!NumObjects)
	  wsprintf(title, "EMPTY");
  else
	  wsprintf(title, "%d of %d", Current + 1, NumObjects);

  BrowseButtons->SetBarCaption(title);


}


void Browser::CmBeginning() {

  if (NumObjects && CanMove()){
	 Current = 0;
	 Display();
	 UpdateStatusBar();
  }

}

void Browser::CeBeginning(TCommandEnabler& ce){
  ce.Enable(NumObjects);
}


void Browser::CmPrevious(){
  if (NumObjects && CanMove())
	 if (Current > 0){
		 Current--;
		 Display();
		 UpdateStatusBar();
	 }

}

void Browser::CePrevious(TCommandEnabler& ce){
  ce.Enable(NumObjects);
}

void Browser::CmNext() {
  if (NumObjects && CanMove())
	 if (Current < NumObjects - 1){
		 Current++;
		 Display();
		 UpdateStatusBar();
	 }

}

void Browser::CeNext(TCommandEnabler& ce){
  ce.Enable(NumObjects);
}

void Browser::CmLast(){

  if (NumObjects && CanMove()){
	 Current = NumObjects - 1;
	 Display();
	 UpdateStatusBar();
  }

}
void Browser::CeLast(TCommandEnabler& ce){
  ce.Enable(NumObjects);
}



void Browser::BrowserDelete() {

  NumObjects--;
  if (!NumObjects){  // Deleted last item in list ?
	 Current = 0;
	 Clear();
  }
  else
	 if (Current == NumObjects)
		Current--;   // Delted the item on the end of the list so point to
					  // the new last

  Display();    // display the new current
  UpdateStatusBar(); // Update object count


}

void Browser::BrowserSearch(int NewCurrent) {

  if (NewCurrent < NumObjects && NewCurrent > -1){
	 Current = NewCurrent;
	 Display();
	 UpdateStatusBar();
  }
}

void Browser::BrowserAdd(int newcurrent){


  NumObjects++;
  newcurrent >= 0 && newcurrent < NumObjects ? Current = newcurrent
														 : Current = NumObjects - 1;

  Display();
  UpdateStatusBar();


}





