// Browser.h
// By Paul DuBois 9-15-95
// Free for non comercial use


#if !defined(BROWSER_H)
#define BROWSER_H



#include <owl\floatfra.h>

// I stole this code from bc45\examples\owl\owlapp\paint
// A slightly customized floating frame for use as our tool & style button
// palettes.
//
class TButtonBar : public TFloatingFrame {
  public:
	 TButtonBar(TWindow*        parent,
						 char*           title = 0,
						 TWindow*        clientWnd = 0,
						 TModule*        module = 0);

	 void SetBarCaption(char* cs);

  private:

	 void SetupWindow();
	 void CleanupWindow();
	 void EvClose();

	 // include these event handler shells here so the buttons are enabled
		void CmBeginning(){};
		void CmPrevious(){};
		void CmNext(){};
		void CmLast(){};
		void CmHelp(){};

  DECLARE_RESPONSE_TABLE(TButtonBar);
};
class _OWLCLASS Browser{

  public:
	 Browser(){NumObjects = 0; Current = 0;}

	 ~Browser(){}

	 void BegBrowse(int NewNumObjects); // Call this function to initialize
													  // the browser and display the first
													  // object.  Do not call this function
													  // until the floating tool bar is created

	 void CreateBar(TFrameWindow *parent); // Call this with your frame

  protected:



	 int GetCurrent() {return Current;} // access function
	 int GetNumObjects() {return NumObjects;}

	 // Use BrowserAdd after you add an object.  It updates the browser.
	 // Pass it the index for the current object(The one you added).  It
	 // assumes one object is added per add.
	 void BrowserAdd(int);

	 // Use BrowserDelete after you delete an object.  It updates the browser.
	 // It assumes one object is deleted per delete.
	 void BrowserDelete();

	 // Use BrowserSearch after you perform a search function.  Pass it what
	 // the new current object should be
	 void BrowserSearch(int );

	 virtual bool CanMove(){return true;} //Override this function to implement
													  // saving an object before moving

	 virtual bool Display(){return true;}// display the current object. Return False if
													 // display failed

	 virtual bool Clear(){return false;}       // Clear the object from the screen.  Returns
											// true if successfull.  Used when NumObjects
											// becomes 0 after a delete.

	 // Include response table entries in your derived class for the following
	 // events.  These events are for the navigation through the
	 // objects.  To implement specific actions before updating the browser event
	 // (such as saving the current record), override the function, then
	 // explicity call this one.

	 void CmBeginning();       // Moves to first object and Display()
	 void CeBeginning(TCommandEnabler& ce);
	 void CmPrevious();        // Moves to previous object and Display()
	 void CePrevious(TCommandEnabler& ce);
	 void CmNext();            // Moves to next object and Display()
	 void CeNext(TCommandEnabler& ce);
	 void CmLast();             // Moves to last object and Display()
	 void CeLast(TCommandEnabler& ce);

	 // ViewTools and Enable tools are used to turn the button bar on and off.
	 void CmViewTools();
	 void CeViewTools(TCommandEnabler& ce);

  private:

	 int NumObjects;  // Number of objects in array.  Initialized by child
	 int Current;     // Current object.  Initialized by child

	 // Used to update the caption in the button bar and to create or destroy the bar
		TButtonBar *BrowseButtons;
		void UpdateStatusBar();  // Update the bar caption (3 of 99)


};
      
#endif // if !defined(BROWSER_H
